#include "simul.h"

#define MAXFIL 32
#define SQRT15 3.87298334620741688517927
#define N_ELEM(st) (sizeof(st)/sizeof((st)[0]))
#define MOD(i, j) ( (i) >= 0 ? (i) % (j) : ((j) - ((-(i)) % (j))) % (j) )
#define MAXBAND        256
#define FORWARD    	1
#define BACKWARD  	-1
#define ALLEVELS  	0
#define BAD 		-1
#define RS		1
#define LS		2
#define US		3
#define DS		4
#define D1              1
#define D2              2

/*DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD*/
/*DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD*/
/*DDDDDDDDDDD                                           DDDDDDDDDDDD*/
/*DDDDDDDDDDD      Definition of waveleet filters       DDDDDDDDDDDD*/
/*DDDDDDDDDDD                                           DDDDDDDDDDDD*/
/*DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD*/
/*DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD*/

typedef struct {
	float *cH;
	int nH;
	float *cHtilde;
	int nHtilde;
	int offH, offG, offHtilde, offGtilde;
} waveletfilter;




/*
 *	-----------------------------------------------------------------
 *	Battle-Lemarie filter
 *
 *	Source: Mallat, "A Theory for Multiresolution Signal Decomposition: The
 *	  Wavelet Representation", IEEE PAMI, v. 11, no. 7, 674-693, Table 1
 */

static float cHBattleLemarie[] = {
	M_SQRT2 * -0.002,
	M_SQRT2 * -0.003,
	M_SQRT2 *  0.006,
	M_SQRT2 *  0.006,
	M_SQRT2 * -0.013,
	M_SQRT2 * -0.012,
	M_SQRT2 *  0.030,  /*  5 and 6 sign change from Mallat's paper */
	M_SQRT2 *  0.023,
	M_SQRT2 * -0.078,
	M_SQRT2 * -0.035,
	M_SQRT2 *  0.307,
	M_SQRT2 *  0.542,
	M_SQRT2 *  0.307,
	M_SQRT2 * -0.035,
	M_SQRT2 * -0.078,
	M_SQRT2 *  0.023,
	M_SQRT2 *  0.030,
	M_SQRT2 * -0.012,
	M_SQRT2 * -0.013,
	M_SQRT2 *  0.006,
	M_SQRT2 *  0.006,
	M_SQRT2 * -0.003,
	M_SQRT2 * -0.002,
	  0.0
};

waveletfilter wfltrBattleLemarie = {
	cHBattleLemarie,
	N_ELEM(cHBattleLemarie),
	cHBattleLemarie,
	N_ELEM(cHBattleLemarie),
	/* offsets of H, G, Htilde, and Gtilde, respectively */
	11, 11, 11, 11
};

/*
 *	-----------------------------------------------------------------
 *	Burt-Adelson filter
 *
 *	Source: TLoW, Table 8.4
 */
static float cHBurtAdelson[] = {
	M_SQRT2 * -1.0 / 20.0,
	M_SQRT2 *  5.0 / 20.0,
	M_SQRT2 * 12.0 / 20.0,
	M_SQRT2 *  5.0 / 20.0,
	M_SQRT2 * -1.0 / 20.0,
	  0.0
};
static float cHtildeBurtAdelson[] = {
	  0.0,
	M_SQRT2 *  -3.0 / 280.0,
	M_SQRT2 * -15.0 / 280.0,
	M_SQRT2 *  73.0 / 280.0,
	M_SQRT2 * 170.0 / 280.0,
	M_SQRT2 *  73.0 / 280.0,
	M_SQRT2 * -15.0 / 280.0,
	M_SQRT2 *  -3.0 / 280.0
};
waveletfilter wfltrBurtAdelson = {
	cHBurtAdelson,
	N_ELEM(cHBurtAdelson),
	cHtildeBurtAdelson,
	N_ELEM(cHtildeBurtAdelson),
	/* offsets of H, G, Htilde, and Gtilde, respectively */
	2, 2, 4, 2
};

/*
 *	-----------------------------------------------------------------
 *	Coiflet filters
 *
 *	Source: Beylkin, Coifman, and Rokhlin "Fast Wavelet Transforms and
 *	  Numerical Algorithms I", Comm. Pure Appl. Math, v. 44, Appendix A
 */

static float cHCoiflet_2[] = {
	M_SQRT2 * (SQRT15 - 3) / 32.0,
	M_SQRT2 * (1 - SQRT15) / 32.0,
	M_SQRT2 * (6 - 2 * SQRT15) / 32.0,
	M_SQRT2 * (2 * SQRT15 + 6) / 32.0,
	M_SQRT2 * (SQRT15 + 13) / 32.0,
	M_SQRT2 * (9 - SQRT15) / 32.0
};
waveletfilter wfltrCoiflet_2 = {
	cHCoiflet_2,
	N_ELEM(cHCoiflet_2),
	cHCoiflet_2,
	N_ELEM(cHCoiflet_2),
	/* offsets of H, G, Htilde, and Gtilde, respectively */
	3, 1, 3, 1
};

static float cHCoiflet_4[] = {
	 0.0011945726958388,
	-0.01284557955324,
	 0.024804330519353,
	 0.050023519962135,
	-0.15535722285996,
	-0.071638282295294,
	 0.57046500145033,
	 0.75033630585287,
	 0.28061165190244,
	-0.0074103835186718,
	-0.014611552521451,
	-0.0013587990591632
};
waveletfilter wfltrCoiflet_4 = {
	cHCoiflet_4,
	N_ELEM(cHCoiflet_4),
	cHCoiflet_4,
	N_ELEM(cHCoiflet_4),
	/* offsets of H, G, Htilde, and Gtilde, respectively */
	6, 4, 6, 4
};

static float cHCoiflet_6[] = {
	-0.0016918510194918,
	-0.00348787621998426,
	 0.019191160680044,
	 0.021671094636352,
	-0.098507213321468,
	-0.056997424478478,
	 0.45678712217269,
	 0.78931940900416,
	 0.38055713085151,
	-0.070438748794943,
	-0.056514193868065,
	 0.036409962612716,
	 0.0087601307091635,
	-0.011194759273835,
	-0.0019213354141368,
	 0.0020413809772660,
	 0.00044583039753204,
	-0.00021625727664696
};
waveletfilter wfltrCoiflet_6 = {
	cHCoiflet_6,
	N_ELEM(cHCoiflet_6),
	cHCoiflet_6,
	N_ELEM(cHCoiflet_6),
	/* offsets of H, G, Htilde, and Gtilde, respectively */
	6, 10, 6, 10
};

/*
 *	-----------------------------------------------------------------
 *	Daubechies filters
 *
 *	Source: TLoW, Table 6.1
 */

#ifndef SQRT3
#define SQRT3 1.73205080756887729352745
#endif

static float cHDaubechies_4[] = {
	M_SQRT2 * (1 + SQRT3) / 8.0,
	M_SQRT2 * (3 + SQRT3) / 8.0,
	M_SQRT2 * (3 - SQRT3) / 8.0,
	M_SQRT2 * (1 - SQRT3) / 8.0
};
waveletfilter wfltrDaubechies_4 = {
	cHDaubechies_4,
	N_ELEM(cHDaubechies_4),
	cHDaubechies_4,
	N_ELEM(cHDaubechies_4),
	/* offsets of H, G, Htilde, and Gtilde, respectively */
	1, 1, 1, 1
};

static float cHDaubechies_6[] = {
	 0.332670552950,
	 0.806891509311,
	 0.459877502118,
	-0.135011020010,
	-0.085441273882,
	 0.035226291882
};
waveletfilter wfltrDaubechies_6 = {
	cHDaubechies_6,
	N_ELEM(cHDaubechies_6),
	cHDaubechies_6,
	N_ELEM(cHDaubechies_6),
	/* offsets of H, G, Htilde, and Gtilde, respectively */
	1, 3, 1, 3
};

static float cHDaubechies_8[] = {
	 0.230377813309,
     0.714846570553,
	 0.6308807667930,
	-0.027983769417,
	-0.187034811719,
	 0.030841381836,
	 0.032883011667,
	-0.010597401785
};
waveletfilter wfltrDaubechies_8 = {
	cHDaubechies_8,
	N_ELEM(cHDaubechies_8),
	cHDaubechies_8,
	N_ELEM(cHDaubechies_8),
	/* offsets of H, G, Htilde, and Gtilde, respectively */
	1, 5, 1, 5
};

static float cHDaubechies_10[] = {
	 0.1601023979741929,
	 0.6038292697971895,
	 0.7243085284377726,
	 0.1384281459013203,
	-0.2422948870663823,
	-0.0322448695846381,
	 0.0775714938400459,
	-0.0062414902127983,
	-0.0125807519990820,
	 0.0033357252854738
};
waveletfilter wfltrDaubechies_10 = {
	cHDaubechies_10,
	N_ELEM(cHDaubechies_10),
	cHDaubechies_10,
	N_ELEM(cHDaubechies_10),
	/* offsets of H, G, Htilde, and Gtilde, respectively */
	1, 7, 1, 7
};

static float cHDaubechies_12[] = {
	 0.1115407433501095,
	 0.4946238903984533,
	 0.7511339080210959,
	 0.3152503517091982,
	-0.2262646939654400,
	-0.1297668675672625,
	 0.0975016055873225,
	 0.0275228655303053,
	-0.0315820393184862,
	 0.0005538422011614,
	 0.0047772575119455,
	-0.0010773010853085
};
waveletfilter wfltrDaubechies_12 = {
	cHDaubechies_12,
	N_ELEM(cHDaubechies_12),
	cHDaubechies_12,
	N_ELEM(cHDaubechies_12),
	/* offsets of H, G, Htilde, and Gtilde, respectively */
	1, 9, 1, 9
};

static float cHDaubechies_20[] = {
	 0.026670057901,
	 0.188176800078,
	 0.527201188932,
	 0.688459039454,
	 0.281172343661,
	-0.249846424327,
	-0.195946274377,
	 0.127369340336,
	 0.093057364604,
	-0.071394147166,
	-0.029457536822,
	 0.033212674059,
	 0.003606553567,
	-0.010733175483,
	 0.001395351747,
	 0.001992405295,
	-0.000685856695,
	-0.000116466855,
	 0.000093588670,
	-0.000013264203
};
waveletfilter wfltrDaubechies_20 = {
	cHDaubechies_20,
	N_ELEM(cHDaubechies_20),
	cHDaubechies_20,
	N_ELEM(cHDaubechies_20),
	/* offsets of H, G, Htilde, and Gtilde, respectively */
	2, 16, 2, 16
};


/*
 *	-----------------------------------------------------------------
 *	Haar filter
 *
 *	Source: TLoW, p. 10 (and lots of other places!)
 */

static float cHHaar[] = {
	M_SQRT2 * 1.0 / 2.0,
	M_SQRT2 * 1.0 / 2.0
};

waveletfilter wfltrHaar = {
	cHHaar,
	N_ELEM(cHHaar),
	cHHaar,
	N_ELEM(cHHaar),
	/* offsets of H, G, Htilde, and Gtilde, respectively */
	0, 0, 0, 0
};

/*
 *	-----------------------------------------------------------------
 *	Pseudocoiflet filter
 *	Source: Reissell, "Multiresolution Geometric Algorithms Using Wavelets I:
 *	  Representation for Parametric Curves and Surfaces", UBC TR 93-17, p. 33
 */

static float cHPseudocoiflet_4[] = {
	M_SQRT2 *  -1.0 / 512.0,
	  0.0,
	M_SQRT2 *  18.0 / 512.0,
	M_SQRT2 * -16.0 / 512.0,
	M_SQRT2 * -63.0 / 512.0,
	M_SQRT2 * 144.0 / 512.0,
	M_SQRT2 * 348.0 / 512.0,
	M_SQRT2 * 144.0 / 512.0,
	M_SQRT2 * -63.0 / 512.0,
	M_SQRT2 * -16.0 / 512.0,
	M_SQRT2 *  18.0 / 512.0,
	  0.0,
	M_SQRT2 *  -1.0 / 512.0,
	  0.0
};
static float cHtildePseudocoiflet_4[] = {
	  0.0,
	M_SQRT2 *  -1.0 / 32.0,
	  0.0,
	M_SQRT2 *   9.0 / 32.0,
	M_SQRT2 *  16.0 / 32.0,
	M_SQRT2 *   9.0 / 32.0,
	  0.0,
	M_SQRT2 *  -1.0 / 32.0
};
waveletfilter wfltrPseudocoiflet_4_4 = {
	cHPseudocoiflet_4,
	N_ELEM(cHPseudocoiflet_4),
	cHtildePseudocoiflet_4,
	N_ELEM(cHtildePseudocoiflet_4),
	/* offsets of H, G, Htilde, and Gtilde, respectively */
	6, 2, 4, 6
};

/*
 *	-----------------------------------------------------------------
 *	Spline filters
 *
 *	Source: TLoW, Table 8.2 (corrected)
 */
static float cHSpline_2[] = {
	M_SQRT2 * -0.125,
	M_SQRT2 *  0.25,
	M_SQRT2 *  0.75,
	M_SQRT2 *  0.25,
	M_SQRT2 * -0.125,
	  0.0
};
static float cHSpline_3[] = {
	M_SQRT2 * 1.0 / 8.0,
	M_SQRT2 * 3.0 / 8.0,
	M_SQRT2 * 3.0 / 8.0,
	M_SQRT2 * 1.0 / 8.0
};
static float cHSpline_4[] = {
	M_SQRT2 *   3.0 / 128.0,
	M_SQRT2 *  -6.0 / 128.0,
	M_SQRT2 * -16.0 / 128.0,
	M_SQRT2 *  38.0 / 128.0,
	M_SQRT2 *  90.0 / 128.0,
	M_SQRT2 *  38.0 / 128.0,
	M_SQRT2 * -16.0 / 128.0,
	M_SQRT2 *  -6.0 / 128.0,
	M_SQRT2 *   3.0 / 128.0,
	   0.0
};
static float cHtildeSpline_2[] = {
	   0.0,
	M_SQRT2 * 1.0 / 4.0,
	M_SQRT2 * 2.0 / 4.0,
	M_SQRT2 * 1.0 / 4.0
};
static float cHtildeSpline_3[] = {
	M_SQRT2 *  3.0 / 64.0,
	M_SQRT2 * -9.0 / 64.0,
	M_SQRT2 * -7.0 / 64.0,
	M_SQRT2 * 45.0 / 64.0,
	M_SQRT2 * 45.0 / 64.0,
	M_SQRT2 * -7.0 / 64.0,
	M_SQRT2 * -9.0 / 64.0,
	M_SQRT2 *  3.0 / 64.0
};
static float cHtildeSpline_7[] = {
	M_SQRT2 *   -35.0 / 16384.0,
	M_SQRT2 *  -105.0 / 16384.0,
	M_SQRT2 *  -195.0 / 16384.0,
	M_SQRT2 *   865.0 / 16384.0,
	M_SQRT2 *   363.0 / 16384.0,	/* corrected ("363" was "336") in text) */
	M_SQRT2 * -3489.0 / 16384.0,
	M_SQRT2 *  -307.0 / 16384.0,
	M_SQRT2 * 11025.0 / 16384.0,
	M_SQRT2 * 11025.0 / 16384.0,
	M_SQRT2 *  -307.0 / 16384.0,
	M_SQRT2 * -3489.0 / 16384.0,
	M_SQRT2 *   363.0 / 16384.0,	/* corrected ("363" was "336") in text) */
	M_SQRT2 *   865.0 / 16384.0,
	M_SQRT2 *  -195.0 / 16384.0,
	M_SQRT2 *  -105.0 / 16384.0,
	M_SQRT2 *   -35.0 / 16384.0
};

waveletfilter wfltrSpline_2_2 = {
	cHSpline_2,
	N_ELEM(cHSpline_2),
	cHtildeSpline_2,
	N_ELEM(cHtildeSpline_2),
	/* offsets of H, G, Htilde, and Gtilde, respectively */
	2, 0, 2, 2
};
waveletfilter wfltrSpline_2_4 = {
	cHSpline_4,
	N_ELEM(cHSpline_4),
	cHtildeSpline_2,
	N_ELEM(cHtildeSpline_2),
	/* offsets of H, G, Htilde, and Gtilde, respectively */
	4, 0, 2, 4
};
waveletfilter wfltrSpline_3_3 = {
	cHSpline_3,
	N_ELEM(cHSpline_3),
	cHtildeSpline_3,
	N_ELEM(cHtildeSpline_3),
	/* offsets of H, G, Htilde, and Gtilde, respectively */
	1, 3, 3, 1
};
waveletfilter wfltrSpline_3_7 = {
	cHSpline_3,
	N_ELEM(cHSpline_3),
	cHtildeSpline_7,
	N_ELEM(cHtildeSpline_7),
	/* offsets of H, G, Htilde, and Gtilde, respectively */
	1, 7, 7, 1
};

struct poss_filters {waveletfilter *filter;
					 char *name;
					 char *abr;
					 }wfilters[]={
	{&wfltrBattleLemarie,	 "BattleLemarie", 	 "bl"  },
	{&wfltrBurtAdelson,	     "BurtAdelson",	 	 "ba"  },
	{&wfltrCoiflet_2,	     "Coiflet_2",	 	 "c2"  },
	{&wfltrCoiflet_4,	     "Coiflet_4",	 	 "c4"  },
	{&wfltrCoiflet_6,	     "Coiflet_6",	 	 "c6"  },
	{&wfltrDaubechies_4,	 "Daubechies_4",	 "d4"  },
	{&wfltrDaubechies_6,	 "Daubechies_6",	 "d6"  },
	{&wfltrDaubechies_8,	 "Daubechies_8",	 "d8"  },
	{&wfltrDaubechies_10,    "Daubechies_10",	 "d10" },
	{&wfltrDaubechies_12,    "Daubechies_12",	 "d12" },
	{&wfltrDaubechies_20,    "Daubechies_20",	 "d20" },
	{&wfltrHaar,	         "Haar",			 "ha"  },
	{&wfltrPseudocoiflet_4_4,"Pseudocoiflet_4_4","pc44"},
	{&wfltrSpline_2_2,	     "Spline_2_2",		 "s22" },
	{&wfltrSpline_2_4,	     "Spline_2_4",		 "s24" },
	{&wfltrSpline_3_3,	     "Spline_3_3",		 "s33" },
	{&wfltrSpline_3_7,	     "Spline_3_7",		 "s37" }};


int numfil=N_ELEM(wfilters);
waveletfilter *actfil;
/*---------------------------------------------------------------------------------------*/
/* Allocate memory  float type                                                           */
/*---------------------------------------------------------------------------------------*/
flalloc(array,size)
float **array;
long size;
{if( ((*array)=(float *) calloc(size,sizeof(float))) ==0 )
    {printf("Problem to allocate memory for you. Why ?\n");exit(0);}
 }

/*---------------------------------------------------------------------------------------*/
/* Orders coeficients in array according to hilbert curve                                */
/*---------------------------------------------------------------------------------------*/
hilb_recurs(data,vx,vy,posx,posy,pdata,count,blsize,type,smer)
  float *data[],*pdata;
  int vx,vy,blsize,type,*count,smer,posx,posy;
{ if(blsize==1)
    { if(smer==FORWARD){pdata[*count]=*(data[posy]+posx);(*count)++;}
      else   {*(data[posy]+posx)=pdata[*count];(*count)++;};
      }
  else
    {blsize/=2;
     switch(type)
       {case(RS):hilb_recurs(data,vx,vy,posx       ,posy       ,pdata,count,blsize,DS,smer);
                 hilb_recurs(data,vx,vy,posx+blsize,posy       ,pdata,count,blsize,RS,smer);
                 hilb_recurs(data,vx,vy,posx+blsize,posy+blsize,pdata,count,blsize,RS,smer);
                 hilb_recurs(data,vx,vy,posx       ,posy+blsize,pdata,count,blsize,US,smer);
                 break;
        case(LS):hilb_recurs(data,vx,vy,posx+blsize,posy+blsize,pdata,count,blsize,US,smer);
                 hilb_recurs(data,vx,vy,posx       ,posy+blsize,pdata,count,blsize,LS,smer);
                 hilb_recurs(data,vx,vy,posx       ,posy       ,pdata,count,blsize,LS,smer);
                 hilb_recurs(data,vx,vy,posx+blsize,posy       ,pdata,count,blsize,DS,smer);
                 break;
        case(US):hilb_recurs(data,vx,vy,posx+blsize,posy+blsize,pdata,count,blsize,LS,smer);
                 hilb_recurs(data,vx,vy,posx+blsize,posy       ,pdata,count,blsize,US,smer);
                 hilb_recurs(data,vx,vy,posx       ,posy       ,pdata,count,blsize,US,smer);
                 hilb_recurs(data,vx,vy,posx       ,posy+blsize,pdata,count,blsize,RS,smer);
                 break;
        case(DS):hilb_recurs(data,vx,vy,posx       ,posy       ,pdata,count,blsize,RS,smer);
                 hilb_recurs(data,vx,vy,posx       ,posy+blsize,pdata,count,blsize,DS,smer);
                 hilb_recurs(data,vx,vy,posx+blsize,posy+blsize,pdata,count,blsize,DS,smer);
                 hilb_recurs(data,vx,vy,posx+blsize,posy       ,pdata,count,blsize,LS,smer);
                 break;
        };
   }
 };
 /*****************************************************************/
/*pomocny vypis prvkov zadaneho pola                             */
void wav_vypis(komentar,pole,pocet)
   char *komentar;
   float *pole;
   int pocet;
{int a;
 printf ("%s {",komentar);
 for(a=0;a<pocet;a++)
   printf("%6.2f, ",pole[a]);
 printf("}\n");
 }

/*******************************************************************/
/* robi priamu konvoluciu, tj. vystupne pole bude polovicnej dlzky */
void dekom1d(input,filter,output,inlen,fillen,posun)
float *input,*filter,*output;
int inlen,fillen,posun;
{int i,j;
 for(j=0;j<inlen/2;j++) *(output+j)=0;
 for(j=0;j<inlen/2;j++)
   for(i=0;i<fillen;i++)
	 *(output+j)+= *(filter+i) * *(input+ MOD( 2*j + i -posun,inlen) );
 }
/******************************************************************/
/* robi spatnu konvoluciu, tj. vystupne pole bude 2nasobnej dlzky */
void rekon1d(input,filter,output,inlen,fillen,posun)
float *input,*filter,*output;
int inlen,fillen,posun;
{int i,j;
 int outlen=2*inlen;
 for(j=0;j<outlen;j++)*(output+j)=0;
 for(j=0;j<inlen;j++)
   for(i=0;i<fillen;i++)
	 *(output+MOD(2*j+i-posun,outlen)) += *(filter+i) * *(input+j);
 }
/*****************************************************************/
/* vyraba mirror filter zo zadaneho filtra (NIE PODLA DEFINICIE) */
void mirror(zdroj,ciel,pocet)
float *zdroj,*ciel;
int pocet;
{int a,znam= -1;
 for(a=0;a<pocet;a++)
   {*(ciel+a)= znam* *(zdroj + MOD(pocet -a -1,pocet));
	znam *= -1;
	}
 }
/*****************************************************************/
/* vykona priamu Wavelet transfofrmaciu                          */
void DWT(data,dlen,levels)
float *data;
int dlen,levels;
{float pomoc[MAX1D];
 float mirfil[MAXFIL];
 int size,j,stopval;
 mirror(actfil->cHtilde,mirfil,actfil->nHtilde);
 if(levels==ALLEVELS)stopval=1;
 else stopval=dlen>>levels;
 for(size=dlen;size>stopval;size /= 2)
   {dekom1d(data,actfil->cH,pomoc,size,actfil->nH,actfil->offH);
	dekom1d(data,mirfil,pomoc+size/2,size,actfil->nHtilde,actfil->offG);
	memcpy(data,pomoc,size*sizeof(float));
	}
 }
/*****************************************************************/
/* vykona inverznu Wavelet transfofrmaciu                        */
void IDWT(data,dlen,levels)
float *data;
int dlen,levels;
{float pomoc[2*MAX1D];
 float mirfil[MAXFIL];
 int size,a,startval;
 mirror(actfil->cH,mirfil,actfil->nH);
 if(levels==ALLEVELS)startval=1;
 else startval= dlen>>levels ;
 for(size=startval;size<dlen;size *= 2)
   {rekon1d(data,actfil->cHtilde,pomoc,size,actfil->nHtilde,actfil->offHtilde);
	rekon1d(data+size,mirfil,pomoc+2*size,size,actfil->nH,actfil->offGtilde);
	for(a=0;a<2*size;a++)
	  *(data+a) = ( *(pomoc+a) + *(pomoc+2*size+a));
	}
 }
/*-----------------------------------------------------------------------*/
/* 2d wavelet trafo pre smer x   standard					 */
void  xwav(d,vx,vy,smer)
	 float *d[];
	 int vx,vy,smer;
{int a,b;
 static float pomoc[MAX1D];
 for(a=0;a<vy;a++)
   {for(b=0;b<vx;b++)
	 *(pomoc+b)= *(d[a]+b);
	 if(smer==1)DWT(pomoc,vx,ALLEVELS);
	 else IDWT(pomoc,vx,ALLEVELS);
	for(b=0;b<vx;b++)
	  *(d[a]+b)= *(pomoc+b);
	}
}
/*-----------------------------------------------------------------------*/
/* 2d wavelet trafo pre smer y   standard				         */
void  ywav(d,vx,vy,smer)
	 float *d[];
	 int vx,vy,smer;
{int a,b;
 static float pomoc[MAX1D];
 for(a=0;a<vx;a++)
   {for(b=0;b<vy;b++)
	 *(pomoc+b)= *(d[b]+a);
	 if(smer==1)DWT(pomoc,vy,ALLEVELS);
	 else IDWT(pomoc,vy,ALLEVELS);
	for(b=0;b<vy;b++)
	  *(d[b]+a)= *(pomoc+b);
	}
}
/*-----------------------------------------------------------------------*/
/* 2D wavelet trafo NON-standard				         */
void  nwav(d,vx,vy,smer)
	 float *d[];
	 int vx,vy,smer;
{int i,b,sx,sy;
 float pomoc[MAX1D];
 
 if(smer==1)
   for(sx=vx,sy=vy;sx>1&&sy>1;sx/=2,sy/=2)
      { /* x smer */
        for(i=0;i<sy;i++) 
          {for(b=0;b<sx;b++) *(pomoc+b)= *(d[i]+b);
           DWT(pomoc,sx,1);
           for(b=0;b<sx;b++) *(d[i]+b)= *(pomoc+b);
           }
       /* y smer*/
        for(i=0;i<sx;i++)
          {for(b=0;b<sy;b++) *(pomoc+b)= *(d[b]+i);
           DWT(pomoc,sy,1);
           for(b=0;b<sy;b++) *(d[b]+i)= *(pomoc+b);
           }
       }
 else /*spatny smer*/
   {int insx,insy;
    if(vx>=vy) {insx=2*vx/vy;insy=2;}
    else      {insx=2;insy=2*vy/vx;}
    for(sx=insx,sy=insy;sx<=vx&&sy<=vy;sx * =2,sy * =2)
      { /* x smer */
        for(i=0;i<sy;i++) 
          {for(b=0;b<sx;b++) *(pomoc+b)= *(d[i]+b);
           IDWT(pomoc,sx,1);
           for(b=0;b<sx;b++) *(d[i]+b)= *(pomoc+b);
           }
       /* y smer*/
        for(i=0;i<sx;i++)
          {for(b=0;b<sy;b++) *(pomoc+b)= *(d[b]+i);
           IDWT(pomoc,sy,1);
           for(b=0;b<sy;b++) *(d[b]+i)= *(pomoc+b);
           }
       }
   }
  }
/*****************************************************************/
/* vykona priamu Wavelet transfofrmaciu    velke pole            */
void SwapDWT(data,dlen,levels)
float *data;
int dlen,levels;
{float *pomoc;
 float mirfil[MAXFIL];
 int size,j,stopval;
 flalloc(&pomoc,dlen);
 mirror(actfil->cHtilde,mirfil,actfil->nHtilde);
 if(levels==ALLEVELS)stopval=1;
 else stopval=dlen>>levels;
 for(size=dlen;size>stopval;size /= 2)
   {dekom1d(data,actfil->cH,pomoc,size,actfil->nH,actfil->offH);
	dekom1d(data,mirfil,pomoc+size/2,size,actfil->nHtilde,actfil->offG);
	memcpy(data,pomoc,size*sizeof(float));
	}
 free(pomoc);
 }
/*****************************************************************/
/* vykona inverznu Wavelet transfofrmaciu   velke pole           */
void SwapIDWT(data,dlen,levels)
float *data;
int dlen,levels;
{float *pomoc;
 float mirfil[MAXFIL];
 int size,a,startval;
 flalloc(&pomoc,dlen*2);
 mirror(actfil->cH,mirfil,actfil->nH);
 if(levels==ALLEVELS)startval=1;
 else startval= dlen>>levels ;
 for(size=startval;size<dlen;size *= 2)
   {rekon1d(data,actfil->cHtilde,pomoc,size,actfil->nHtilde,actfil->offHtilde);
	rekon1d(data+size,mirfil,pomoc+2*size,size,actfil->nH,actfil->offGtilde);
	for(a=0;a<2*size;a++)
	  *(data+a) = ( *(pomoc+a) + *(pomoc+2*size+a));
	}
 free(pomoc);
 }
/*---------------------------------------------------------------------------------------*/
/* Perform 2d wavelet trafo via 1d case and hilbert scanning                             */
/*---------------------------------------------------------------------------------------*/
void  hwav(data,vx,vy,smer)
	 float *data[];
	 int vx,vy,smer;
{float *pdata;
 int a,b,count=0;
 flalloc(&pdata,vx*vy);
 if (smer==FORWARD)
   { hilb_recurs(data,vx,vx,0,0,pdata,&count,vx,DS,FORWARD);
     SwapDWT(pdata,vx*vy,ALLEVELS);
     for(a=0;a<vy;a++)
       memcpy(data[a],pdata+vx*a,vx*sizeof(float));
     } 
  else
   { for(a=0;a<vy;a++)
       memcpy(pdata+vx*a,data[a],vx*sizeof(float));
     SwapIDWT(pdata,vx*vy,ALLEVELS);
     hilb_recurs(data,vx,vx,0,0,pdata,&count,vx,DS,BACKWARD);
      
     };
 }






